package com.flyer.exception;

import cn.dev33.satoken.exception.NotLoginException;
import com.flyer.enums.StatusCodeEnum;
import com.flyer.util.R;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.ServletRequestBindingException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.stream.Collectors;

/**
 * 全局异常处理
 *
 * @author kelei
 * @since 2022/4/2/15:18
 */
@Slf4j
@RestControllerAdvice
public class GlobalExceptionHandler {
    @ExceptionHandler(value = MethodArgumentNotValidException.class)
    public R handleVaildException(MethodArgumentNotValidException e) {
        log.error(e.getMessage(), e);
        BindingResult bindingResult = e.getBindingResult();
        StringBuilder errMsgBuilder = new StringBuilder();
        for (FieldError fieldError : bindingResult.getFieldErrors()) {
            errMsgBuilder.append(fieldError.getDefaultMessage());
            break;
        }
        return R.error().message(errMsgBuilder.toString());
    }

    @ExceptionHandler(value = ConstraintViolationException.class)
    public R handleVaildException(ConstraintViolationException e) {
        log.error(e.getMessage(), e);
        String message = e.getConstraintViolations().stream().map(ConstraintViolation::getMessage).collect(Collectors.joining());
        return R.error().message(message);
    }

    @ExceptionHandler(value = {
            Throwable.class,
            ServletRequestBindingException.class
    })
    public R handleException(Throwable throwable) {
        log.error(throwable.getMessage(), throwable);
        return R.error(StatusCodeEnum.ERROR);
    }

    @ExceptionHandler(value = BusinessException.class)
    public R handleException(BusinessException e) {
        log.error(e.getMessage(), e);
        return R.error().message(e.getMessage()).code(e.getCode());
    }

    @ExceptionHandler(value = NotLoginException.class)
    public R handleException(NotLoginException e) {
        log.error(e.getMessage(), e);
        String msg = "";
        switch (e.getType()) {
            case "-1":
            case "-2":
                msg = "无效的token";
                break;
            case "-3":
                msg = "token已过期";
                break;
            case "-4":
                break;
            case "-5":
                break;
            case "-6":
                break;
            case "-7":
                break;
        }
        return R.error().message(msg).code(e.getCode());
    }
}
